@@ -1,3 +1,4 @@
+#include "osp-preamble.h"
 #include "osperl.h"
 
 #define TV_PANIC		croak
@@ -11,7 +12,10 @@
 #define FREE_TN(tn)		delete tn
 
 /* You should not call these directly */
-#define NEW_TCE(near,xx) \
-	new(os_segment::of(near), TCE::get_os_typespec(), xx) TCE[xx]
+#define NEW_TCE(ret, near,xx) \
+	NEW_OS_ARRAY(ret, os_segment::of(near), TCE::get_os_typespec(), TCE, xx)
+
+//	ret = new(os_segment::of(near), TCE::get_os_typespec(), xx) TCE[xx]
+
 #define FREE_TCE(tce)		delete [] tce
 
